package cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.entry;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

/**
 * 合同审批 创建 Request VO
 *
 * @author 符溶馨
 */
@Schema(description = "管理后台 - 入职申请创建 Request VO")
@Data
public class BpmOAEntryCreateReqVO {

    @Schema(description = "新入职用户部门编号", example = "10237")
    private Long entryDeptId;

    @Schema(description = "新入职用户岗位编号", example = "10237")
    private Long entryPostId;

    @Schema(description = "用户昵称", requiredMode = Schema.RequiredMode.REQUIRED, example = "王五")
    @NotNull(message = "用户昵称不能为空")
    private String nickname;

    @Schema(description = "合同公司 | 字典值参考bpm_entry_company", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "合同公司不能为空")
    private Integer contractCompany;

    @Schema(description = "手机号码", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "手机号码不能为空")
    private String mobile;

    @Schema(description = "身份证号码")
    private String idcard;

    @Schema(description = "入职时间", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "入职时间不能为空")
    @DateTimeFormat(pattern = DateUtils.FORMAT_YEAR_MONTH_DAY)
    private LocalDate entryDate;

    @Schema(description = "面试评价", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "面试评价不能为空")
    private String interviewEvaluation;

    @Schema(description = "薪资情况", example = "27005")
    private String salary;

    @Schema(description = "流程实例编号")
    private String processInstanceId;

    @Schema(description = "状态-参见 bpm_process_instance_result 枚举", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    private Integer result;

    @Schema(description = "上传文件", requiredMode = Schema.RequiredMode.REQUIRED)
    private List<UploadUserFile> fileItems;

}
