package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.type.JsonLongSetTypeHandler;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Set;

/**
 * OA 奖惩申请 DO
 *
 * @author 符溶馨
 */
@TableName(value ="bpm_oa_incentive", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAIncentiveDO extends BaseDO {


    /**
     * 出差表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     */
    private Long userId;

    /**
     * 奖惩类型
     * 1:奖励 2:惩罚
     */
    private Integer type;

    /**
     * 奖惩人用户编号
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> incentiveUserIds;

    /**
     * 奖惩人部门编号
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> incentiveDeptIds;

    /**
     * 奖惩时间
     */
    private LocalDateTime time;

    /**
     * 奖惩金额
     */
    private BigDecimal amount;

    /**
     * 奖惩事由
     */
    private String reason;

    /**
     * 用章的结果
     *
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;
}
