package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.reimbursement.Reimbursement;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * OA 报销申请 DO
 *
 * @author 姚君
 *
 */
@TableName(value ="bpm_oa_reimbursement", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAReimbursementDO extends BaseDO {

    /**
     * 1: 自费
     */
    public static final Integer EXPENSE_TYPE = 1;
    /**
     * 2: 备用金
     */
    public static final Integer IMPREST_TYPE = 2;

    /**
     * 请假表单主键
     */
    @TableId
    private Long id;
    /**
     * 申请人的用户编号
     *
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 收款人名称
     */
    private String nickname;

    /**
     * 收款人卡号
     */
    private String bankNo;

    /**
     * 开户行信息
     */
    private String bankName;

    /**
     * 报销明细数据JSON
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<Reimbursement> reimbursements;
    /**
     * 总报销金额
     */
    private BigDecimal totalMoney;

    /**
     * 总报销金额的中文大写
     */
    private String totalMoneyChinese;

    /**
     * 报销类型
     */
    private Integer reimbursementType;

    /**
     * 备用差额
     */
    private BigDecimal difference;

    /**
     * 报销发票总张数
     */
    private Integer totalQuantity;

    /**
     * 报销的结果
     *
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems;

}
