package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.worktask.BpmOAWorkTaskTrackInfo;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.time.LocalDate;
import java.util.List;

/**
 * OA 分配任务 DO
 *
 * @author 姚君
 
 */
@TableName(value ="bpm_oa_work_task", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAWorkTaskDO extends BaseDO {

    /**
     * 工作任务表单主键
     */
    @TableId
    private Long id;

    /**
     * 发起人
     */
    private Long fromUserId;

    /**
     * 发起人部门编号
     */
    private Long fromDeptId;

    /**
     * 接受人
     */
    private Long receiverUserId;

    /**
     * 任务名称
     */
    private String taskName;
    /**
     * 任务内容
     */
    private String taskContent;
    /**
     * 任务级别
     * 字典值 参考 bpm_oa_work_task_type
     */
    private Integer level;
    /**
     * 开始时间
     */
    private LocalDate startTime;
    /**
     * 截止时间
     */
    private LocalDate endTime;
    /**
     * 任务进度 百分比
     */
    private Long schedule;
    /**
     * 任务的状态  任务自身的状态  1:待启动，2:进行中，3.已完成，4.延期, 5.搁置
     * 字典值 参考 bpm_oa_work_task_status
     */
    private Integer workTaskStatus ;
    /**
     * 搁置或延期 原因
     */
    private String reason;
    /**
     * 搁置后 任务开始日期
     */
    private LocalDate shelveStartTime;
    /**
     * 搁置后 任务结束日期
     */
    private LocalDate shelveEndTime;

    /**
     * 任务分配BPM流程的结果
     *
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;

    /**
     * 任务跟踪信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<BpmOAWorkTaskTrackInfo> trackInfo;

}
