package cn.iocoder.yudao.module.bpm.service.financialpayment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.controller.admin.financialpayment.vo.FinancialPaymentPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.financialpayment.vo.FinancialPaymentSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.financialpayment.vo.FinancialPaymentSaveVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.financialpayment.FinancialPaymentDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 财务支付管理 Service 接口
 *
 * @author 艾楷
 */
public interface FinancialPaymentService {

    /**
     * 创建财务支付管理
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createFinancialPayment(@Valid FinancialPaymentSaveVO vo);

    /**
     * 更新财务支付管理
     *
     * @param updateReqVO 更新信息
     */
    void updateFinancialPayment(@Valid FinancialPaymentSaveReqVO updateReqVO);

    /**
     * 删除财务支付管理
     *
     * @param id 编号
     */
    void deleteFinancialPayment(Long id);

    /**
     * 获得财务支付管理
     *
     * @param id 编号
     * @return 财务支付管理
     */
    FinancialPaymentDO getFinancialPayment(Long id);

    /**
     * 获得财务支付管理分页
     *
     * @param pageReqVO 分页查询
     * @return 财务支付管理分页
     */
    PageResult<FinancialPaymentDO> getFinancialPaymentPage(FinancialPaymentPageReqVO pageReqVO);

    /**
     * 插入财务支付
     *
     * @param financialPaymentDO
     */
    void save(FinancialPaymentDO financialPaymentDO);

    /**
     * 领取
     *
     * @param ids
     */
    void receive(List<Long> ids);

    /**
     * 更新
     *
     * @param financialPayment
     */
    void updateById(FinancialPaymentDO financialPayment);

    /**
     * 取消领取
     *
     * @param ids
     */
    void cancel(List<Long> ids);
}
