package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.regular.BpmOARegularCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOARegularConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOARegularDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOARegularMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_REGULAR_NOT_EXISTS;

/**
 * OA 转正申请 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOARegularServiceImpl extends BpmOABaseService implements BpmOARegularService{

    /**
     * OA 转正对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_regular";

    @Resource
    private BpmOARegularMapper regularMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createRegular(Long userId, BpmOARegularCreateReqVO createReqVO) {

        //插入OA 转正申请
        BpmOARegularDO regular = BpmOARegularConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        regularMapper.insert(regular) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(regular.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 转正单中
        regularMapper.updateById(new BpmOARegularDO().setId(regular.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return regular.getId();
    }

    @Override
    public void updateRegularResult(Long id, Integer result) {

        validateLeaveExists(id);
        regularMapper.updateById(new BpmOARegularDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (regularMapper.selectById(id) == null) {
            throw exception(OA_REGULAR_NOT_EXISTS);
        }
    }

    @Override
    public BpmOARegularDO getRegular(Long id) {

        return regularMapper.selectById(id);
    }

    @Override
    public BpmOARegularDO getByProcessInstanceId(String processInstanceId) {

        return regularMapper.selectOne(BpmOARegularDO::getProcessInstanceId, processInstanceId);
    }
}
