package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.seal.BpmOASealCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOASealConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOASealDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOASealMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_SEAL_NOT_EXISTS;

/**
 * OA 用章申请 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOASealServiceImpl extends BpmOABaseService implements BpmOASealService {

    /**
     * OA 用章对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_seal";

    @Resource
    private BpmOASealMapper sealMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createSeal(Long userId, BpmOASealCreateReqVO createReqVO) {

        //插入OA 用章申请
        BpmOASealDO seal = BpmOASealConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        sealMapper.insert(seal) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(seal.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 用章单中
        sealMapper.updateById(new BpmOASealDO().setId(seal.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return seal.getId();
    }

    @Override
    public void updateSealResult(Long id, Integer result) {

        validateLeaveExists(id);
        sealMapper.updateById(new BpmOASealDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (sealMapper.selectById(id) == null) {
            throw exception(OA_SEAL_NOT_EXISTS);
        }
    }

    @Override
    public BpmOASealDO getSeal(Long id) {

        return sealMapper.selectById(id);
    }

    @Override
    public BpmOASealDO getByProcessInstanceId(String processInstanceId) {

        return sealMapper.selectOne(BpmOASealDO::getProcessInstanceId, processInstanceId);
    }
}
