package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.shiftjobs.BpmOAShiftjobsCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAShiftjobsConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAShiftjobsDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAShiftjobsMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_SHIFTJOBS_NOT_EXISTS;

/**
 * OA 调岗申请 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOAShiftjobsServiceImpl extends BpmOABaseService implements BpmOAShiftjobsService{

    /**
     * OA 调岗对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_shiftjobs";

    @Resource
    private BpmOAShiftjobsMapper shiftjobsMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createShiftjobs(Long userId, BpmOAShiftjobsCreateReqVO createReqVO) {

        //插入OA 调岗申请
        BpmOAShiftjobsDO shiftjobs = BpmOAShiftjobsConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        shiftjobsMapper.insert(shiftjobs) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(shiftjobs.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 调岗单中
        shiftjobsMapper.updateById(new BpmOAShiftjobsDO().setId(shiftjobs.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return shiftjobs.getId();
    }

    @Override
    public void updateShiftjobsResult(Long id, Integer result) {

        validateLeaveExists(id);
        shiftjobsMapper.updateById(new BpmOAShiftjobsDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (shiftjobsMapper.selectById(id) == null) {
            throw exception(OA_SHIFTJOBS_NOT_EXISTS);
        }
    }

    @Override
    public BpmOAShiftjobsDO getShiftjobs(Long id) {

        return shiftjobsMapper.selectById(id);
    }

    @Override
    public BpmOAShiftjobsDO getByProcessInstanceId(String processInstanceId) {

        return shiftjobsMapper.selectByProcessInstanceId(processInstanceId);
    }
}
