package cn.iocoder.yudao.module.infra.api.file;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.infra.api.file.dto.FileCreateReqDTO;
import cn.iocoder.yudao.module.infra.api.file.dto.UserFileUpdateReqDTO;
import cn.iocoder.yudao.module.infra.enums.ApiConstants;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@FeignClient(name = ApiConstants.NAME) // TODO 芋艿：fallbackFactory =
@Tag(name = "RPC 服务 - 文件")
public interface FileApi {

    String PREFIX = ApiConstants.PREFIX + "/file";

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param content 文件内容
     * @return 文件路径
     */
    default String createFile(byte[] content) {
        return createFile(null, null, content);
    }

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    default String createFile(String path, byte[] content) {
        return createFile(null, path, content);
    }

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param name 原文件名称
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    default String createFile(@RequestParam("name") String name,
                              @RequestParam("path") String path,
                              @RequestParam("content") byte[] content) {
        return createFile(new FileCreateReqDTO().setName(name).setPath(path).setContent(content)).getCheckedData();
    }

    @PostMapping(PREFIX + "/create")
    @Operation(summary = "保存文件，并返回文件的访问路径")
    CommonResult<String> createFile(@Valid @RequestBody FileCreateReqDTO createReqDTO);

    @PostMapping(PREFIX + "/uploadBpmFileProcessInstanceId")
    @Operation(summary = "保存文件，并返回文件的访问路径")
    CommonResult<String> uploadBpmFileProcessInstanceId(@RequestBody FileCreateReqDTO createReqDTO);

    @PostMapping(PREFIX + "/getUserSignImgPath")
    @Operation(summary = "获取用户的签名图片地址")
    CommonResult<String> getUserSignImgPath(@RequestParam("userId") Long userId);

    @PostMapping(PREFIX + "/createBusinessFile")
    @Operation(summary = "保存业务附件， 并返回文件的访问路径")
    String createBusinessFile(@RequestParam("bussinessType")Long bussinessType, @RequestParam("name") String name, @RequestBody byte[] content) ;

    @PostMapping(PREFIX + "/updateBusinessFileContent")
    @Operation(summary = "修改业务附件infra_file_content的content字段")
    String updateBusinessFileContent(@RequestParam("url") String url,
                                     @RequestParam("businessType") Long businessType,
                                     @RequestParam("name") String name,
                                     @RequestBody byte[] content);

    @PostMapping(PREFIX + "/updateBusinessFile")
    @Operation(summary = "修改BusinessFile文件中的 businessInstanceId | 入职申请用")
    CommonResult<Boolean> updateBusinessFileFormEntry(@RequestParam("url") String url,
                                                      @RequestParam("businessInstanceId") String businessInstanceId);


    @DeleteMapping(PREFIX + "/deleteBpmFile")
    @Operation(summary = "删除工作流附件")
    @Parameter(name = "url", description = "附件URL地址", required = true)
    CommonResult<Boolean> deleteBpmFile(@RequestParam("url") String url) throws Exception;

    @PostMapping(PREFIX + "/updateUserFile")
    @Operation(summary = "修改用户文件绑定 用户编号")
    CommonResult<Boolean> updateUserFileUserId(@RequestBody UserFileUpdateReqDTO updateReqVO);
}
