package cn.iocoder.yudao.module.infra.service.file;

import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.BpmFileUploadReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.BusinessFileUploadReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FilePageReqVO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.UserFileUpdateReqVO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * 文件 Service 接口
 *

 */
public interface FileService {

    /**
     * 获得文件分页
     *
     * @param pageReqVO 分页查询
     * @return 文件分页
     */
    PageResult<FileDO> getFilePage(FilePageReqVO pageReqVO);

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param name 文件名称
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    String createFile(String name, String path, byte[] content);

    /**
     * 保存文件，并返回文件对象
     *
     * @param file 上传文件对象
     * @return 文件路径
     */
    BpmFileDO createReturnFile(MultipartFile file);

    /**
     * 删除文件
     *
     * @param id 编号
     */
    void deleteFile(Long id) throws Exception;

    /**
     * 获得文件内容
     *
     * @param configId 配置编号
     * @param path 文件路径
     * @return 文件内容
     */
    byte[] getFileContent(Long configId, String path) throws Exception;


    /**
     * 删除文件
     *
     * @param url 文件URL地址
     */
    void deleteFile(String url) throws Exception;

    /**
     * 删除bpm文件
     * @param url
     * @throws Exception
     */
    void deleteBpmFile(String url) throws Exception;

    /**
     * 更新文件的流程实例ID
     * @param reqVO
     * @throws Exception
     */
    void uploadBpmFileProcessInstanceId(BpmFileUploadReqVO reqVO) throws  Exception ;

    /**
     * 保存业务类型文件，并返回文件对象
     *
     * @param file 上传文件对象
     * @param businessType 业务类型
     * @return 文件路径
     */
    BusinessFileDO createBusinessReturnFile(MultipartFile file, Long businessType);

    /**
     * 删除bpm文件
     * @param url
     * @throws Exception
     */
    void deleteBusinessFile(String url) throws Exception;

    /**
     * 更新文件的流程实例ID
     * @param reqVO
     * @throws Exception
     */
    void uploadBusinessFileProcessInstanceId(BusinessFileUploadReqVO reqVO) throws  Exception ;

    /**
     * 更新文件的流程实例ID  | 入职申请用
     * @param url 文件地址
     */
    void uploadBusinessFileFormEntry(String url, String businessInstanceId);

    /**
     * 获取用户的签名图片地址
     *
     * @param userId
     */
    String getUserSignImgPath( Long userId) ;


    /**
     * 保存业务类型的附件
     * @param bussinessType
     * @param name
     * @param content
     * @return
     */
    String createBusinessFile(Long bussinessType, String name, byte[] content) ;

    /**
     * 修改业务类型的附件内容
     * @param url
     * @param content
     * @return
     */
    String updateBusinessFileContent(String url, Long businessType, String name, byte[] content) ;

    /**3
     * 保存用户文件，并返回文件对象
     *
     * @param file 上传文件对象
     * @return 文件路径
     */
    UserFileDO createUserReturnFile(MultipartFile file);

    /**
     * 删除用户文件
     * @param url 文件URL地址
     */
    void deleteUserFile(String url);

    /**
     * 更新用户文件 UserId
     * @param updateReqVO 更新信息
     */
    void updateUserFileUserId(UserFileUpdateReqVO updateReqVO);

    /**
     * 保存业务类型的附件
     * @param name 文件名
     * @param content 字节流
     * @return url
     */
    QRCodeDO updateQRCodeFile(Long id, Long deptId, String name, byte[] content) ;

    /**
     * 获得小程序码信息
     * @param id 编号
     * @return 小程序码信息
     */
    QRCodeDO getQRCode(Long id);

    /**
     * 获得小程序码信息
     * @param deptId 部门编号
     * @param scene 参数
     * @return 小程序码信息
     */
    QRCodeDO getQRCode(Long deptId, String scene);

    /**
     * 更新 小程序码生成时间
     * @param id 编号
     */
    void updateQRCodeFile(Long id);

    /**
     * 删除小程序码文件
     * @param url 文件链接
     */
    void deleteQRCode(String url);

    /**
     * 创建小程序码文件
     * @param deptId 部门编号
     * @param scene 参数
     */
    Long createQRCodeFile(Long deptId, String scene);
}
