package cn.iocoder.yudao.module.system.dal.mysql.user;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.system.controller.admin.user.dto.UserPageDTO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.factoryUser.FactoryUserPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserRespVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserSimpleRespVO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

@Mapper
public interface AdminUserMapper extends BaseMapperX<AdminUserDO> {

    default AdminUserDO sgetByOpenId(String openId) {
        return selectOne(AdminUserDO::getOpenId, openId);
    }

    default AdminUserDO selectByUsername(String username) {
        return selectOne(AdminUserDO::getUsername, username);
    }

    default AdminUserDO selectByEmail(String email) {
        return selectOne(AdminUserDO::getEmail, email);
    }

    default AdminUserDO selectByMobile(String mobile) {
        return selectOne(AdminUserDO::getMobile, mobile);
    }

    default PageResult<AdminUserDO> selectPage(UserPageReqVO reqVO, Collection<Long> deptIds) {
        return selectPage(reqVO, new LambdaQueryWrapperX<AdminUserDO>()
                .likeIfPresent(AdminUserDO::getUsername, reqVO.getUsername())
                .likeIfPresent(AdminUserDO::getMobile, reqVO.getMobile())
                .likeIfPresent(AdminUserDO::getNickname, reqVO.getNickName())
                .eqIfPresent(AdminUserDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(AdminUserDO::getCreateTime, reqVO.getCreateTime())
                .inIfPresent(AdminUserDO::getDeptId, deptIds)
                .eq(AdminUserDO::getUserType, 1)    //查询公司用户
                .orderByDesc(AdminUserDO::getId));
    }

    default PageResult<AdminUserDO> selectFactoryUserPage(FactoryUserPageReqVO reqVO, Long deptId) {

        return selectPage(reqVO, new LambdaQueryWrapperX<AdminUserDO>()
                .likeIfPresent(AdminUserDO::getNickname, reqVO.getNickname())
                .eqIfPresent(AdminUserDO::getDeptId, deptId)
                .eq(AdminUserDO::getUserType, 2)
                .orderByDesc(AdminUserDO::getId));
    }

    default List<AdminUserDO> selectListByNickname(String nickname) {
        return selectList(new LambdaQueryWrapperX<AdminUserDO>().like(AdminUserDO::getNickname, nickname));
    }

    default List<AdminUserDO> selectListByStatus(Integer status) {

        return selectList(new LambdaQueryWrapperX<AdminUserDO>()
                .eq(AdminUserDO::getStatus, status)
                .eq(AdminUserDO::getUserType, 1));
    }

    default List<AdminUserDO> selectListByDeptIds(Collection<Long> deptIds) {

        return selectList(new LambdaQueryWrapperX<AdminUserDO>()
                .in(AdminUserDO::getDeptId, deptIds)
                .eq(AdminUserDO::getUserType, 1));
    }

    default List<AdminUserDO> selectListByDeptId(Long deptId, Long userId) {
        return selectList(new LambdaQueryWrapperX<AdminUserDO>()
                .eq(AdminUserDO::getDeptId, deptId)
                .ne(AdminUserDO::getId, userId)
                .eq(AdminUserDO::getUserType, 1));
    }

    void emptyOpenId(@Param("openId") String openId);

    List<UserRespVO> selectByDeptIds(Collection<Long> deptIds);

    /**
     * 根据部门ids查询出用户列表 过滤掉考勤组里有的
     *
     * @param deptIds
     * @param groupId
     * @return
     */
    List<UserRespVO> selectByDeptIdsFilterGroupUser(@Param("deptIds") Collection<Long> deptIds, @Param("groupId") Long groupId);

    List<Long> selectUserByBoss();

    /**
     * 获取考勤组里所有的用户
     *
     * @param mpPage
     * @param dto
     * @return
     */
    IPage<UserSimpleRespVO> getAllUserListByGroupIds(@Param("mpPage") IPage mpPage, @Param("dto") UserPageDTO dto);
}
