package cn.iocoder.yudao.module.system.service.assets;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.assets.vo.AssetsTypePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.assets.vo.AssetsTypeSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.assets.AssetsTypeDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 资产类型 Service 接口
 *
 * @author 艾楷
 */
public interface AssetsTypeService {

    /**
     * 创建资产类型
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createAssetsType(@Valid AssetsTypeSaveReqVO createReqVO);

    /**
     * 更新资产类型
     *
     * @param updateReqVO 更新信息
     */
    void updateAssetsType(@Valid AssetsTypeSaveReqVO updateReqVO);

    /**
     * 删除资产类型
     *
     * @param id 编号
     */
    void deleteAssetsType(Long id);

    /**
     * 获得资产类型
     *
     * @param id 编号
     * @return 资产类型
     */
    AssetsTypeDO getAssetsType(Long id);

    /**
     * 获得资产类型分页
     *
     * @param pageReqVO 分页查询
     * @return 资产类型分页
     */
    PageResult<AssetsTypeDO> getAssetsTypePage(AssetsTypePageReqVO pageReqVO);

    /**
     * 根据pid获取列表
     *
     * @param pid
     * @return
     */
    List<AssetsTypeDO> getByPid(Long pid);

    /**
     * 获取全部
     *
     * @return
     */
    List<AssetsTypeDO> getAllList();

    /**
     * 根据资产名称 获得指定资产类型信息
     * @param name 资产名称
     * @return 资产类型
     */
    AssetsTypeDO getByName(String name);
}