package cn.iocoder.yudao.module.system.service.attendance.fixed;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.Constants;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.attendance.dto.AttendancePunchPageDTO;
import cn.iocoder.yudao.module.system.controller.admin.attendance.vo.AttendancePunchPageVO;
import cn.iocoder.yudao.module.system.controller.admin.attendance.vo.AttendanceRulesVO;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.AttendanceFixedPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.AttendanceFixedRespVO;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.AttendanceFixedSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.fixed.AttendanceFixedDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.fixed.AttendanceFixedMapper;
import cn.iocoder.yudao.module.system.service.attendance.AttendanceService;
import cn.iocoder.yudao.module.system.service.attendance.groupshift.AttendanceGroupShiftService;
import cn.iocoder.yudao.module.system.service.attendance.punch.PunchService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.FIXED_NOT_EXISTS;

/**
 * 固定班制考勤设置 Service 实现类
 *
 * @author 艾楷
 */
@Service("fixed")
@Validated
@Slf4j
public class AttendanceFixedServiceImpl implements AttendanceFixedService, PunchService {
    @Resource
    private AttendanceFixedMapper attendanceFixedMapper;
    @Resource
    @Lazy // 避免依赖循环
    private AttendanceService attendanceService;
    @Resource
    private AttendanceGroupShiftService attendanceGroupShiftService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long batchCreateOrUpdate(Long attendanceGroupId, List<AttendanceFixedSaveReqVO> vos) {
        Long groupId = null;
        // 插入
        List<AttendanceFixedDO> oldList = attendanceFixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .eq(AttendanceFixedDO::getAttendanceGroupId, attendanceGroupId));
        List<AttendanceFixedDO> fixedList = BeanUtils.toBean(vos, AttendanceFixedDO.class);
        // -- 如果旧的没数据 则直接新增
        if (oldList.isEmpty()) {
            groupId = attendanceGroupId;
            attendanceFixedMapper.insertBatch(fixedList);
        } else {
            //如果旧的有数据则区分下
            // 插入
            List<AttendanceFixedDO> saveList = fixedList.stream().filter(a -> a.getId() == null).collect(Collectors.toList());
            List<AttendanceFixedDO> editList = fixedList.stream().filter(a -> a.getId() != null).collect(Collectors.toList());
            if (!saveList.isEmpty()) {
                groupId = attendanceGroupId;
                attendanceFixedMapper.insertBatch(saveList);
            }
            LocalDateTime now = LocalDateTime.now();
            int week = now.getDayOfWeek().getValue();
            if (!editList.isEmpty()) {
                Map<Integer, AttendanceFixedDO> map = oldList.stream().collect(Collectors.toMap(AttendanceFixedDO::getWeekTime, a -> a));
                for (AttendanceFixedDO attendanceFixedDO : editList) {
                    AttendanceFixedDO item = map.get(attendanceFixedDO.getWeekTime());
                    boolean eqFlag = BeanUtil.isCommonFieldsEqual(attendanceFixedDO, item, "createTime", "updateTime", "creator", "updater", "deleted");
                    // -- 判断当天是周几是否和当前修改的时间为同一天 如果是的话则判断则提示立即生效
                    if (!eqFlag && week == attendanceFixedDO.getWeekTime()) {
                        groupId = attendanceGroupId;
                        break;
                    }
                }
                attendanceFixedMapper.updateBatch(editList);
            }
            // -- 需要删除的 - 
            List<Long> oldIds = oldList.stream().map(AttendanceFixedDO::getId).collect(Collectors.toList());
            List<Long> newIds = editList.stream().map(AttendanceFixedDO::getId).collect(Collectors.toList());
            List<Long> delIds = CollectionUtil.subtractToList(oldIds, newIds);
            if (!delIds.isEmpty()) {
                groupId = attendanceGroupId;
                attendanceFixedMapper.deleteBatchIds(delIds);
            }
        }
        return groupId;
    }

    @Override
    public void updateFixed(AttendanceFixedSaveReqVO updateReqVO) {
        // 校验存在
        validateFixedExists(updateReqVO.getId());
        // 更新
        AttendanceFixedDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceFixedDO.class);
        attendanceFixedMapper.updateById(updateObj);
    }

    @Override
    public void deleteFixed(Long id) {
        // 校验存在
        validateFixedExists(id);
        // 删除
        attendanceFixedMapper.deleteById(id);
    }

    private void validateFixedExists(Long id) {
        if (attendanceFixedMapper.selectById(id) == null) {
            throw exception(FIXED_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceFixedDO getFixed(Long id) {
        return attendanceFixedMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceFixedDO> getFixedPage(AttendanceFixedPageReqVO pageReqVO) {
        return attendanceFixedMapper.selectPage(pageReqVO);
    }

    @Override
    public AttendanceFixedDO getByGroupIdAndWeek(Long attendanceGroupId, Integer week) {
        List<AttendanceFixedDO> list = attendanceFixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .eq(attendanceGroupId != null, AttendanceFixedDO::getAttendanceGroupId, attendanceGroupId)
                .eq(week != null, AttendanceFixedDO::getWeekTime, week));
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public List<AttendanceFixedDO> getByGroupId(Long attendanceGroupId) {
        return attendanceFixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .eq(attendanceGroupId != null, AttendanceFixedDO::getAttendanceGroupId, attendanceGroupId));
    }

    @Override
    public List<AttendanceFixedDO> getByGroupIdAndWeek(List<Long> attendanceGroupIds, Integer week) {
        return attendanceFixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .in(CollectionUtil.isNotEmpty(attendanceGroupIds), AttendanceFixedDO::getAttendanceGroupId, attendanceGroupIds)
                .eq(week != null, AttendanceFixedDO::getWeekTime, week));
    }

    @Override
    public Map<Long, Long> getGroupToShiftIdMap(List<AttendanceGroupDO> fixedList, LocalDateTime localDateTime) {
        Map<Long, Long> map = new HashMap<>();
        //获取到当天是周几
        int week = localDateTime.getDayOfWeek().getValue();
        List<Long> groupIds = fixedList.stream().map(AttendanceGroupDO::getId).collect(Collectors.toList());
        List<AttendanceFixedDO> attendanceFixedDOS = this.getByGroupIdAndWeek(groupIds, week);
        Map<Long, List<AttendanceFixedDO>> attendanceFixedMap = attendanceFixedDOS.stream().collect(Collectors.groupingBy(AttendanceFixedDO::getAttendanceGroupId));
        for (AttendanceGroupDO activationGroup : fixedList) {
            // -- 判断是否根据节假日自动排班 - 如果是的话 - 根据排班的来
            Boolean isHolidayFlag = Constants.TRUE.equals(activationGroup.getAutoHolidaysFlag()) ?
                    attendanceService.isHoliday(localDateTime) : null;
            // -- 当前是节假日 并且是放假
            if (isHolidayFlag != null && isHolidayFlag) {
                continue;
            }
            AttendanceFixedDO attendanceFixedDO = null;
            List<AttendanceFixedDO> list = attendanceFixedMap.get(activationGroup.getId());
            if (CollectionUtil.isNotEmpty(list)) {
                attendanceFixedDO = list.get(0);
            }
            // -- 当前没有班次 - 不需要考勤
            if (attendanceFixedDO == null || attendanceFixedDO.getAttendanceGroupShiftId() == null) {
                continue;
            }
            map.put(activationGroup.getId(), attendanceFixedDO.getAttendanceGroupShiftId());
        }
        return map;
    }

    @Override
    public List<AttendanceFixedRespVO> getListByGroupId(Long attendanceGroupId) {
        List<AttendanceFixedRespVO> respVOS = new ArrayList<>();
        List<AttendanceFixedDO> fixedDOList = attendanceFixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .eq(attendanceGroupId != null, AttendanceFixedDO::getAttendanceGroupId, attendanceGroupId));
        if (!fixedDOList.isEmpty()) {
            respVOS = BeanUtils.toBean(fixedDOList, AttendanceFixedRespVO.class);
            List<Long> attendanceGroupShiftIds = fixedDOList.stream().map(AttendanceFixedDO::getAttendanceGroupShiftId).filter(Objects::nonNull).collect(Collectors.toList());
            List<AttendanceGroupShiftVO> vos = attendanceGroupShiftService.listByShiftIds(attendanceGroupShiftIds);
            Map<Long, AttendanceGroupShiftVO> map = vos.stream().collect(Collectors.toMap(AttendanceGroupShiftVO::getId, a -> a));
            for (AttendanceFixedRespVO respVO : respVOS) {
                respVO.setShiftVO(map.get(respVO.getAttendanceGroupShiftId()));
            }
        }
        return respVOS;
    }

    @Override
    public List<Long> getUseGroupIds(Long shiftId) {
        return attendanceFixedMapper.getUseGroupIds(shiftId);
    }

    @Override
    public AttendancePunchPageVO getPunchPage(AttendancePunchPageDTO dto) {
        AttendancePunchPageVO vo = new AttendancePunchPageVO();
        vo.setActivationGroup(dto.getActivationGroup());
        AttendanceGroupDO activationGroup = dto.getActivationGroup();
        // -- 判断是否根据节假日自动排班 - 如果是的话 - 根据排班的来
        Boolean isHolidayFlag = Constants.TRUE.equals(activationGroup.getAutoHolidaysFlag()) ?
                attendanceService.isHoliday(dto.getLocalDateTime()) : null;
        // -- 当前是节假日 并且是放假
        if (isHolidayFlag != null && isHolidayFlag) {
            return vo.setTodayNeedAttendance(Constants.FALSE);
        }
        //获取到当天是周几
        int week = dto.getLocalDateTime().getDayOfWeek().getValue();
        AttendanceFixedDO attendanceFixedDO = this.getByGroupIdAndWeek(activationGroup.getId(), week);
        // -- 当前没有班次 - 不需要考勤
        if (attendanceFixedDO == null || attendanceFixedDO.getAttendanceGroupShiftId() == null) {
            return vo.setTodayNeedAttendance(Constants.FALSE);
        }
        vo.setAttendanceGroupShiftId(attendanceFixedDO.getAttendanceGroupShiftId());
        attendanceService.calculatePunch(dto, vo);
        vo.setAttendanceGroupId(activationGroup.getId());
        vo.setUser(dto.getUser());
        return vo;
    }

    @Override
    public List<AttendanceRulesVO.WorkRules> getRule(Long attendanceGroupId) {
        List<AttendanceRulesVO.WorkRules> rules = new ArrayList<>();
        List<AttendanceFixedDO> list = this.getByGroupId(attendanceGroupId);
        List<Long> attendanceGroupShiftIds = list.stream().map(AttendanceFixedDO::getAttendanceGroupShiftId).filter(Objects::nonNull).collect(Collectors.toList());
        List<AttendanceGroupShiftVO> vos = attendanceGroupShiftService.listByShiftIds(attendanceGroupShiftIds);
        Map<Long, AttendanceGroupShiftVO> map = vos.stream().collect(Collectors.toMap(AttendanceGroupShiftVO::getId, a -> a));
        LocalDateTime now = LocalDateTime.now();
        for (AttendanceFixedDO attendanceFixedDO : list) {
            AttendanceRulesVO.WorkRules workRules = new AttendanceRulesVO.WorkRules();
            String weekday = DateUtils.convertToChineseWeekday(attendanceFixedDO.getWeekTime());
            workRules.setIndexStr(weekday);
            workRules.setFlag(false);
            if (now.getDayOfWeek().getValue() == attendanceFixedDO.getWeekTime()) {
                workRules.setFlag(true);
            }
            if (attendanceFixedDO.getAttendanceGroupShiftId() == null) {
                workRules.setRule("休息");
            } else {
                AttendanceGroupShiftVO attendanceGroupShiftVOS = map.get(attendanceFixedDO.getAttendanceGroupShiftId());
                List<String> itemRule = new ArrayList<>();
                for (AttendanceGroupShiftItemDO attendanceGroupShiftItemDO : attendanceGroupShiftVOS.getAttendanceGroupShiftItemDOList()) {
                    itemRule.add(attendanceGroupShiftItemDO.getBeginTime() + "-" + attendanceGroupShiftItemDO.getEndTime());
                }
                String rule = String.join(",", itemRule);
                workRules.setRule(rule);
            }
            rules.add(workRules);
        }
        return rules;
    }

}