package cn.iocoder.yudao.module.system.service.attendance.group;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.group.vo.AttendanceGroupPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.group.vo.AttendanceGroupSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 考勤组 Service 接口
 *
 * @author 艾楷
 */
public interface AttendanceGroupService {

    /**
     * 创建考勤组
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createGroup(@Valid AttendanceGroupSaveReqVO createReqVO);

    /**
     * 更新考勤组
     *
     * @param updateReqVO 更新信息
     */
    void updateGroup(@Valid AttendanceGroupSaveReqVO updateReqVO);

    /**
     * 删除考勤组
     *
     * @param id 编号
     */
    void deleteGroup(Long id);

    /**
     * 获得考勤组
     *
     * @param id 编号
     * @return 考勤组
     */
    AttendanceGroupDO getGroup(Long id);

    /**
     * 获得考勤组分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤组分页
     */
    PageResult<AttendanceGroupDO> getGroupPage(AttendanceGroupPageReqVO pageReqVO);

    /**
     * 获取用户所在群组
     *
     * @param userId
     * @return
     */
    AttendanceGroupDO getByUserId(Long userId);

    /**
     * 获取所有
     *
     * @return
     */
    List<AttendanceGroupDO> getAll();

    /**
     * 获取使用班次id的考勤组ids
     *
     * @param shiftId
     * @return
     */
    List<Long> getUseGroupIds(Long shiftId);

}