package cn.iocoder.yudao.module.system.service.attendance.groupshift;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshift.AttendanceGroupShiftDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 考勤组班次 Service 接口
 *
 * @author 艾楷
 */
public interface AttendanceGroupShiftService {

    /**
     * 创建考勤组班次
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createGroupShift(@Valid AttendanceGroupShiftSaveReqVO createReqVO);

    /**
     * 更新考勤组班次
     *
     * @param updateReqVO 更新信息
     */
    List<Long> updateGroupShift(@Valid AttendanceGroupShiftSaveReqVO updateReqVO);

    /**
     * 删除考勤组班次
     *
     * @param id 编号
     */
    void deleteGroupShift(Long id);

    /**
     * 获得考勤组班次
     *
     * @param id 编号
     * @return 考勤组班次
     */
    AttendanceGroupShiftDO getGroupShift(Long id);

    /**
     * 获得考勤组班次分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤组班次分页
     */
    PageResult<AttendanceGroupShiftDO> getGroupShiftPage(AttendanceGroupShiftPageReqVO pageReqVO);

    /**
     * 获取所有班次列表
     *
     * @return
     */
    List<AttendanceGroupShiftVO> allList();

    /**
     * 获取班次列表根据班次ids
     *
     * @return
     */
    List<AttendanceGroupShiftVO> listByShiftIds(List<Long> listByShiftIds);

    /**
     * 立即生效
     *
     * @param groupIds
     */
    void effectiveImmediately(List<Long> groupIds);
}