package cn.iocoder.yudao.module.system.service.attendance.groupuser;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.groupuser.vo.AttendanceGroupUserCreateOrDelVO;
import cn.iocoder.yudao.module.system.controller.admin.groupuser.vo.AttendanceGroupUserPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupuser.vo.AttendanceGroupUserSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupuser.AttendanceGroupUserDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 考勤组人员 Service 接口
 *
 * @author 艾楷
 */
public interface AttendanceGroupUserService {

    /**
     * 创建删除考勤组人员
     *
     * @param attendanceGroupId
     * @param userIds
     */
    AttendanceGroupUserCreateOrDelVO createOrDel(Long attendanceGroupId, List<Long> userIds);

    /**
     * 更新考勤组人员
     *
     * @param updateReqVO 更新信息
     */
    void updateGroupUser(@Valid AttendanceGroupUserSaveReqVO updateReqVO);

    /**
     * 删除考勤组人员
     *
     * @param id 编号
     */
    void deleteGroupUser(Long id);

    /**
     * 获得考勤组人员
     *
     * @param id 编号
     * @return 考勤组人员
     */
    AttendanceGroupUserDO getGroupUser(Long id);

    /**
     * 获得考勤组人员分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤组人员分页
     */
    PageResult<AttendanceGroupUserDO> getGroupUserPage(AttendanceGroupUserPageReqVO pageReqVO);

    /**
     * 根据考勤组id获取考勤人员ids
     *
     * @param groupId
     * @return
     */
    List<Long> getUserIdsByGroupId(Long groupId);

    /**
     * 人员调整立即生效
     *
     * @param vo
     */
    void effectiveImmediately(AttendanceGroupUserCreateOrDelVO vo);

    /**
     * 通过考勤组ids获取考勤人员ids
     * @param groupIds
     */
    List<Long> getUserIdsByGroupIds(List<Long> groupIds);
}