package cn.iocoder.yudao.module.system.service.attendance.punchrecord;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.api.attendance.dto.AttendancePunchRecordDTO;
import cn.iocoder.yudao.module.system.controller.admin.punchrecord.vo.AttendancePunchRecordPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.punchrecord.vo.AttendancePunchRecordSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupuser.AttendanceGroupUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.punchrecord.AttendancePunchRecordDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * 用户打卡记录 Service 接口
 *
 * @author 艾楷
 */
public interface AttendancePunchRecordService {

    /**
     * 创建用户打卡记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPunchRecord(@Valid AttendancePunchRecordSaveReqVO createReqVO);

    /**
     * 更新用户打卡记录
     *
     * @param updateReqVO 更新信息
     */
    void updatePunchRecord(@Valid AttendancePunchRecordSaveReqVO updateReqVO);

    /**
     * 删除用户打卡记录
     *
     * @param id 编号
     */
    void deletePunchRecord(Long id);

    /**
     * 获得用户打卡记录
     *
     * @param id 编号
     * @return 用户打卡记录
     */
    AttendancePunchRecordDO getPunchRecord(Long id);

    /**
     * 获得用户打卡记录分页
     *
     * @param pageReqVO 分页查询
     * @return 用户打卡记录分页
     */
    PageResult<AttendancePunchRecordDO> getPunchRecordPage(AttendancePunchRecordPageReqVO pageReqVO);

    /**
     * 批量新增考勤
     *
     * @param attendancePunchRecordDOList
     */
    void saveBatch(List<AttendancePunchRecordDO> attendancePunchRecordDOList);


    /**
     * 预设考勤
     *
     * @param groupId
     * @param userIds
     * @param localDateTime
     */
    void defaultPersistence(Long groupId, List<Long> userIds, LocalDateTime localDateTime);

    /**
     * 预设考勤
     *
     * @param groupIds      考勤组ids
     * @param localDateTime 时间
     */
    void defaultPersistence(List<Long> groupIds, LocalDateTime localDateTime);

    void defaultPersistence(List<AttendanceGroupDO> attendanceGroupDOS, List<AttendanceGroupUserDO> attendanceGroupUserDOS, LocalDateTime localDateTime);

    /**
     * 预设持久化考勤
     *
     * @param map           获取到考勤组 - 班次 key/value 格式
     * @param groupMap      考勤组id分map
     * @param groupUserMap  考勤组id分组用户列表
     * @param itemMaps      考勤班次子表根据考勤班次分组
     * @param time          时间格式 yyyy-MM-dd
     * @param localDateTime 需要预设的时间
     */
    void defaultPersistence(Map<Long, Long> map, Map<Long, AttendanceGroupDO> groupMap, Map<Long, List<Long>> groupUserMap,
                            Map<Long, AdminUserDO> userMap, Map<Long, List<AttendanceGroupShiftItemDO>> itemMaps, String time, LocalDateTime localDateTime);

    /**
     * 获取未提醒列表 获取当天的未打卡并且未提醒记录
     *
     * @return
     */
    List<AttendancePunchRecordDO> getNotReminded(LocalDateTime localDateTime);

    /**
     * 批量更新
     *
     * @param editList
     */
    void batchUpdate(List<AttendancePunchRecordDO> editList);

    /**
     * 请假修改状态
     *
     * @param dto
     */
    void askingForLeaveAfterwardsToModifyAttendance(AttendancePunchRecordDTO dto);
}
