package cn.iocoder.yudao.module.system.service.attendance.scheduling;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.scheduling.vo.AttendanceSchedulingPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.scheduling.vo.AttendanceSchedulingRespVO;
import cn.iocoder.yudao.module.system.controller.admin.scheduling.vo.AttendanceSchedulingSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.scheduling.AttendanceSchedulingDO;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * 排班制考勤设置 Service 接口
 *
 * @author 艾楷
 */
public interface AttendanceSchedulingService {

    /**
     * 创建排班制考勤设置
     *
     * @param attendanceGroupId
     * @param createReqVO       创建信息
     * @return 编号
     */
    Long batchCreateOrUpdate(Long attendanceGroupId, @Valid List<AttendanceSchedulingSaveReqVO> createReqVO);

    /**
     * 更新排班制考勤设置
     *
     * @param updateReqVO 更新信息
     */
    void updateScheduling(@Valid AttendanceSchedulingSaveReqVO updateReqVO);

    /**
     * 删除排班制考勤设置
     *
     * @param id 编号
     */
    void deleteScheduling(Long id);

    /**
     * 获得排班制考勤设置
     *
     * @param id 编号
     * @return 排班制考勤设置
     */
    AttendanceSchedulingDO getScheduling(Long id);

    /**
     * 获得排班制考勤设置分页
     *
     * @param pageReqVO 分页查询
     * @return 排班制考勤设置分页
     */
    PageResult<AttendanceSchedulingDO> getSchedulingPage(AttendanceSchedulingPageReqVO pageReqVO);

    /**
     * 通过班次id和第几天获取排班制考勤设置
     *
     * @param attendanceGroupId
     * @param indexDay
     * @return
     */
    AttendanceSchedulingDO getSchedulingByIndexDay(Long attendanceGroupId, Integer indexDay);

    List<AttendanceSchedulingDO> getSchedulingByGroupId(Long attendanceGroupId);

    Integer getCycleDay(Long attendanceGroupId);

    /**
     * 获取每个排班考勤组 有多少数量(也就是周期多少日)
     *
     * @return
     */
    Map<Long, Map<String, Object>> countGroupId();

    /**
     * 获取考勤组和班次id的map
     *
     * @param schedulingList
     * @param localDateTime
     * @return
     */
    Map<Long, Long> getGroupToShiftIdMap(List<AttendanceGroupDO> schedulingList, LocalDateTime localDateTime);

    /**
     * 根据考勤组id获取排版考勤设置
     *
     * @param attendanceGroupId
     * @return
     */
    List<AttendanceSchedulingRespVO> getListByGroupId(Long attendanceGroupId);

    /**
     * 获取使用的考勤组ids
     * @param shiftId
     * @return
     */
    List<Long> getUseGroupIds(Long shiftId);
}