package cn.iocoder.yudao.module.system.service.bank;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.bank.vo.UserBankPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.bank.vo.UserBankSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.bank.BankDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 用户银行卡信息 Service 接口
 *
 * @author 符溶馨
 */
public interface UserBankService {

    /**
     * 创建用户银行卡信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBank(@Valid UserBankSaveReqVO createReqVO);

    /**
     * 创建用户银行卡信息
     *
     * @param createReqVO 创建信息
     */
    void createPayBank(@Valid UserBankSaveReqVO createReqVO);

    /**
     * 创建用户和银行卡关联信息
     *
     * @param bankId 银行卡信息编号组
     */
    void createUserBank(List<Long> bankId);

    /**
     * 更新银行卡信息
     *
     * @param updateReqVO 更新信息
     */
    void updateBank(@Valid UserBankSaveReqVO updateReqVO);

    /**
     * 删除用户银行卡信息
     *
     * @param id 编号
     */
    void deleteUserBank(Long id);

    /**
     * 获得用户银行卡信息
     *
     * @param bankId 银行卡信息编号
     * @return 银行卡信息
     */
    BankDO getBank(Long bankId);

    /**
     * 获得银行卡信息编号组
     * @param userId 用户ID
     * @return 银行卡信息编号组
     */
    List<Long> getBankIds(Long userId);

    /**
     * 获得银行卡信息分页
     *
     * @param pageReqVO 分页查询
     * @return 银行卡信息分页
     */
    PageResult<BankDO> getBankPageIn(UserBankPageReqVO pageReqVO, List<Long> bankId);

    /**
     * 获得银行卡信息分页
     *
     * @param pageReqVO 分页查询
     * @return 银行卡信息分页
     */
    PageResult<BankDO> getBankPageNotIn(UserBankPageReqVO pageReqVO, List<Long> bankId);

    List<BankDO> getBankListByUserId(Long userId);
}