package cn.iocoder.yudao.module.system.service.equipment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UserExtImportVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtRespVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserImportRespVO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.UsersExtDO;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

/**
 * 用户信息拓展 Service 接口
 *
 * @author 符溶馨
 */
public interface UsersExtService {

    /**
     * 创建用户信息
     * @param createReqVO 创建信息
     */
    void createUsers(UsersExtSaveReqVO createReqVO);

    /**
     * 创建用户信息拓展
     *
     * @param updateDO 创建信息
     * @return 编号
     */
    String createUsersExt(UsersExtDO updateDO, MultipartFile file) throws IOException;

    /**
     * 更新用户信息拓展
     *
     * @param updateReqVO 更新信息
     */
    void updateUsersExt(@Valid UsersExtSaveReqVO updateReqVO);

    /**
     * 修改用户信息
     * @param updateReqVO 更新信息
     */
    void updateUser(UsersExtSaveReqVO updateReqVO);

    /**
     * 批量更新用户 所下发得设备信息
     *
     * @param updateDo 更新信息
     */
    void updateListUsersExt(@Valid List<UsersExtDO> updateDo);

    /**
     * 删除用户人脸信息
     */
    void deleteUser(Long userId);

    /**
     * 获得用户信息拓展
     *
     * @param id 编号
     * @return 用户信息拓展
     */
    UsersExtDO getUsersExt(Long id);

    /**
     * 获得用户信息拓展
     *
     * @param userId 用户编号
     * @return 用户信息拓展
     */
    UsersExtDO getUsersExtByUserId(Long userId);

    /**
     * 获得指定得用户信息列表
     * @param userId 用户编号
     * @return
     */
    List<UsersExtDO> getListByUserId(List<Long> userId);

    /**
     * 获得用户信息拓展分页
     *
     * @param pageReqVO 分页查询
     * @return 用户信息拓展分页
     */
    PageResult<UsersExtRespVO> getUsersExtPage(UsersExtPageReqVO pageReqVO);

    /**
     * 用户人脸信息 导入
     * @param list 人脸信息
     * @param updateSupport
     * @return
     */
    UserImportRespVO importUserList(List<UserExtImportVO> list, Boolean updateSupport);
}