package cn.iocoder.yudao.module.system.service.laborcontract;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.laborcontract.vo.LaborContractPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.laborcontract.vo.LaborContractSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.laborcontract.LaborContractDO;
import cn.iocoder.yudao.module.system.dal.mysql.laborcontract.LaborContractMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LABOR_CONTRACT_NOT_EXISTS;

/**
 * 劳动合同管理 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LaborContractServiceImpl implements LaborContractService {

    @Resource
    private LaborContractMapper laborContractMapper;

    @Override
    public Long createLaborContract(LaborContractSaveReqVO createReqVO) {
        // 插入
        LaborContractDO laborContract = BeanUtils.toBean(createReqVO, LaborContractDO.class);
        laborContractMapper.insert(laborContract);
        // 返回
        return laborContract.getId();
    }

    @Override
    public void updateLaborContract(LaborContractSaveReqVO updateReqVO) {
        // 校验存在
        validateLaborContractExists(updateReqVO.getId());
        // 更新
        LaborContractDO updateObj = BeanUtils.toBean(updateReqVO, LaborContractDO.class);
        laborContractMapper.updateById(updateObj);
    }

    @Override
    public void deleteLaborContract(Long id) {
        // 校验存在
        validateLaborContractExists(id);
        // 删除
        laborContractMapper.deleteById(id);
    }

    private void validateLaborContractExists(Long id) {
        if (laborContractMapper.selectById(id) == null) {
            throw exception(LABOR_CONTRACT_NOT_EXISTS);
        }
    }

    @Override
    public LaborContractDO getLaborContract(Long id) {
        return laborContractMapper.selectById(id);
    }

    @Override
    public PageResult<LaborContractDO> getLaborContractPage(LaborContractPageReqVO pageReqVO) {
        return null;
    }

}