package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.rule.LogRulePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.rule.LogRuleSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogRuleDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 日志规则 Service 接口
 *
 * @author 符溶馨
 */
public interface LogRuleService {

    /**
     * 创建日志规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLogRule(@Valid LogRuleSaveReqVO createReqVO);

    /**
     * 更新日志规则
     *
     * @param updateReqVO 更新信息
     */
    void updateLogRule(@Valid LogRuleSaveReqVO updateReqVO);

    /**
     * 删除日志规则
     *
     * @param id 编号
     */
    void deleteLogRule(Long id);

    /**
     * 删除日志规则
     *
     * @param formId 模板编号
     */
    void deleteLogRuleByFormId(Long formId);

    /**
     * 获得日志规则
     *
     * @param formId 模板编号
     * @return 日志规则
     */
    LogRuleDO getLogRule(Long formId);

    /**
     * 获得日志规则分页
     *
     * @param pageReqVO 分页查询
     * @return 日志规则分页
     */
    PageResult<LogRuleDO> getLogRulePage(LogRulePageReqVO pageReqVO);

    /**
     * 通过规则ids获取规则
     *
     * @param ruleIds 规则编号组
     * @return 规则列表
     */
    List<LogRuleDO> getLogRuleByIds(List<Long> ruleIds);

    /**
     * 通过模版id获取规则
     *
     * @param formId 日志模板编号
     * @return 日志规则
     */
    LogRuleDO getLogRuleByFormId(Long formId);
}