package cn.iocoder.yudao.module.wms.service.productcategoryproperty;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.wms.controller.admin.productcategoryproperty.vo.ProductCategoryPropertyPageReqVO;
import cn.iocoder.yudao.module.wms.controller.admin.productcategoryproperty.vo.ProductCategoryPropertySaveReqVO;
import cn.iocoder.yudao.module.wms.dal.dataobject.productcategoryproperty.ProductCategoryPropertyDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 产品分类属性 Service 接口
 *
 * @author 艾楷
 */
public interface ProductCategoryPropertyService {

    /**
     * 创建产品分类属性
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductCategoryProperty(@Valid ProductCategoryPropertySaveReqVO createReqVO);

    /**
     * 更新产品分类属性
     *
     * @param updateReqVO 更新信息
     */
    void updateProductCategoryProperty(@Valid ProductCategoryPropertySaveReqVO updateReqVO);

    /**
     * 删除产品分类属性
     *
     * @param id 编号
     */
    void deleteProductCategoryProperty(Long id);

    /**
     * 获得产品分类属性
     *
     * @param id 编号
     * @return 产品分类属性
     */
    ProductCategoryPropertyDO getProductCategoryProperty(Long id);

    /**
     * 获得产品分类属性分页
     *
     * @param pageReqVO 分页查询
     * @return 产品分类属性分页
     */
    PageResult<ProductCategoryPropertyDO> getProductCategoryPropertyPage(ProductCategoryPropertyPageReqVO pageReqVO);

    /**
     * 根据产品分类id获取产品属性列表
     *
     * @param productCategoryId
     * @return
     */
    List<ProductCategoryPropertyDO> getByProductCategoryId(Long productCategoryId);

    /**
     * 根据产品分类ids获取产品属性列表
     *
     * @param productCategoryIds
     * @return
     */
    List<ProductCategoryPropertyDO> getByProductCategoryIds(List<Long> productCategoryIds);
}
