package cn.iocoder.yudao.module.wms.service.productcategoryproperty;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.wms.controller.admin.productcategoryproperty.vo.ProductCategoryPropertyPageReqVO;
import cn.iocoder.yudao.module.wms.controller.admin.productcategoryproperty.vo.ProductCategoryPropertySaveReqVO;
import cn.iocoder.yudao.module.wms.dal.dataobject.productcategoryproperty.ProductCategoryPropertyDO;
import cn.iocoder.yudao.module.wms.dal.mysql.productcategoryproperty.ProductCategoryPropertyMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * 产品分类属性 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class ProductCategoryPropertyServiceImpl implements ProductCategoryPropertyService {

    @Resource
    private ProductCategoryPropertyMapper productCategoryPropertyMapper;

    @Override
    public Long createProductCategoryProperty(ProductCategoryPropertySaveReqVO createReqVO) {
        // 插入
        ProductCategoryPropertyDO productCategoryProperty = BeanUtils.toBean(createReqVO, ProductCategoryPropertyDO.class);
        productCategoryPropertyMapper.insert(productCategoryProperty);
        // 返回
        return productCategoryProperty.getId();
    }

    @Override
    public void updateProductCategoryProperty(ProductCategoryPropertySaveReqVO updateReqVO) {
        // 更新
        ProductCategoryPropertyDO updateObj = BeanUtils.toBean(updateReqVO, ProductCategoryPropertyDO.class);
        productCategoryPropertyMapper.updateById(updateObj);
    }

    @Override
    public void deleteProductCategoryProperty(Long id) {
        // 删除
        productCategoryPropertyMapper.deleteById(id);
    }

    @Override
    public ProductCategoryPropertyDO getProductCategoryProperty(Long id) {
        return productCategoryPropertyMapper.selectById(id);
    }

    @Override
    public PageResult<ProductCategoryPropertyDO> getProductCategoryPropertyPage(ProductCategoryPropertyPageReqVO pageReqVO) {
        return productCategoryPropertyMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ProductCategoryPropertyDO> getByProductCategoryId(Long productCategoryId) {
        return productCategoryPropertyMapper.selectList(new LambdaQueryWrapper<ProductCategoryPropertyDO>()
                .eq(ProductCategoryPropertyDO::getProductCategoryId, productCategoryId));
    }

    @Override
    public List<ProductCategoryPropertyDO> getByProductCategoryIds(List<Long> productCategoryIds) {
        if (productCategoryIds.isEmpty()) {
            return Collections.emptyList();
        }
        return productCategoryPropertyMapper.selectList(new LambdaQueryWrapper<ProductCategoryPropertyDO>()
                .in(ProductCategoryPropertyDO::getProductCategoryId, productCategoryIds));
    }

}
