package cn.iocoder.yudao.module.wms.service.qrcode;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.wms.controller.admin.qrcode.vo.QrCodePageReqVO;
import cn.iocoder.yudao.module.wms.controller.admin.qrcode.vo.QrCodeSaveReqVO;
import cn.iocoder.yudao.module.wms.dal.dataobject.qrcode.QrCodeDO;
import cn.iocoder.yudao.module.wms.dal.mysql.qrcode.QrCodeMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

/**
 * 二维码管理 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class QrCodeServiceImpl implements QrCodeService {

    @Resource
    private QrCodeMapper qrCodeMapper;

    @Override
    public Long createQrCode(QrCodeSaveReqVO createReqVO) {
        // 插入
        QrCodeDO qrCode = BeanUtils.toBean(createReqVO, QrCodeDO.class);
        qrCodeMapper.insert(qrCode);
        // 返回
        return qrCode.getId();
    }

    @Override
    public void updateQrCode(QrCodeSaveReqVO updateReqVO) {
        // 更新
        QrCodeDO updateObj = BeanUtils.toBean(updateReqVO, QrCodeDO.class);
        qrCodeMapper.updateById(updateObj);
    }

    @Override
    public void deleteQrCode(Long id) {
        // 删除
        qrCodeMapper.deleteById(id);
    }

    @Override
    public QrCodeDO getQrCode(Long id) {
        return qrCodeMapper.selectById(id);
    }

    @Override
    public PageResult<QrCodeDO> getQrCodePage(QrCodePageReqVO pageReqVO) {
        return qrCodeMapper.selectPage(pageReqVO);
    }

    @Override
    public void batchDeleteQrCode(List<Long> ids) {
        qrCodeMapper.deleteBatchIds(ids);
    }

}
