package cn.iocoder.yudao.module.smartfactory.controller.admin.factoryinfo.vo;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

@Schema(description = "管理后台 - 工厂信息新增/修改 Request VO")
@Data
public class FactoryInfoSaveReqVO {

    @Schema(description = "编号", requiredMode = Schema.RequiredMode.REQUIRED, example = "28794")
    private Long id;

    @Schema(description = "工厂名字", requiredMode = Schema.RequiredMode.REQUIRED, example = "江西省南昌市第一工厂")
    @NotEmpty(message = "工厂名字不能为空")
    private String name;

    @Schema(description = "工厂简称", requiredMode = Schema.RequiredMode.REQUIRED, example = "第一工厂")
    @NotEmpty(message = "工厂简称不能为空")
    private String shortName;

    @Schema(description = "工厂类型 | 字典值 参考 sf_factory_type", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    @NotEmpty(message = "工厂类型不能为空")
    private String type;

    @Schema(description = "工厂简介", example = "这是一个工厂简介")
    private String description;

    @Schema(description = "省id", requiredMode = Schema.RequiredMode.REQUIRED, example = "19490")
    @NotNull(message = "省id不能为空")
    private Long provinceId;

    @Schema(description = "城市id", requiredMode = Schema.RequiredMode.REQUIRED, example = "28566")
    @NotNull(message = "城市id不能为空")
    private Long cityId;

    @Schema(description = "区县id", requiredMode = Schema.RequiredMode.REQUIRED, example = "22761")
    @NotNull(message = "区县id不能为空")
    private Long adCode;

    @Schema(description = "省名称", requiredMode = Schema.RequiredMode.REQUIRED, example = "江西省")
    @NotEmpty(message = "省名称不能为空")
    private String provinceName;

    @Schema(description = "城市名称", requiredMode = Schema.RequiredMode.REQUIRED, example = "南昌市")
    @NotEmpty(message = "城市名称不能为空")
    private String cityName;

    @Schema(description = "区县名称", requiredMode = Schema.RequiredMode.REQUIRED, example = "红谷滩区")
    @NotEmpty(message = "区县名称不能为空")
    private String districtName;

    @Schema(description = "工厂地址")
    private String address;

    @Schema(description = "经度")
    private BigDecimal lng;

    @Schema(description = "纬度")
    private BigDecimal lat;

    @Schema(description = "状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
//    @NotNull(message = "状态不能为空")
    private Integer status;

    @Schema(description = "负责人编号", example = "126")
    private Long leaderUserId;
}