package cn.iocoder.yudao.module.smartfactory.service.device.cameradevice;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.device.cameradevice.vo.CameraDevicePageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.device.cameradevice.vo.CameraDeviceSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.device.cameradevice.vo.CameraUpdateStatusReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.device.cameradevice.CameraDeviceDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 设备-监控摄像头 Service 接口
 *
 * @author 艾楷
 */
public interface CameraDeviceService {

    /**
     * 创建设备-监控摄像头
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCameraDevice(@Valid CameraDeviceSaveReqVO createReqVO);

    /**
     * 更新设备-监控摄像头
     *
     * @param updateReqVO 更新信息
     */
    void updateCameraDevice(@Valid CameraDeviceSaveReqVO updateReqVO);

    /**
     * 删除设备-监控摄像头
     *
     * @param id 编号
     */
    void deleteCameraDevice(Long id);

    /**
     * 获得设备-监控摄像头
     *
     * @param id 编号
     * @return 设备-监控摄像头
     */
    CameraDeviceDO getCameraDevice(Long id);

    /**
     * 获得设备-监控摄像头分页
     *
     * @param pageReqVO 分页查询
     * @return 设备-监控摄像头分页
     */
    PageResult<CameraDeviceDO> getCameraDevicePage(CameraDevicePageReqVO pageReqVO);

    /**
     * 获得指定工厂下的监控摄像头列表
     * @param factoryId 工厂编号
     * @return 监控摄像头列表
     */
    List<CameraDeviceDO> getListCameraByFactoryId(Long factoryId);

    /**
     * 获得指定监控摄像头列表
     * @param factoryIds 工厂编号
     * @return 监控摄像头列表
     */
    List<CameraDeviceDO> getListCameraByStatus(List<Long> factoryIds);

    /**
     * 更新监控设备状态
     * @param updateReqVO 更新信息
     */
    void updateCameraDeviceStatus(CameraUpdateStatusReqVO updateReqVO);
}