package cn.iocoder.yudao.module.smartfactory.service.factorydata;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.*;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeCreateReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeDetailRespVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorydata.FactoryDataDO;

import javax.validation.Valid;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;

/**
 * 工厂出库入库数据 Service 接口
 *
 * @author 艾楷
 */
public interface FactoryDataService {

    /**
     * 创建工厂出库入库数据
     *
     * @param factoryDataDOS 创建信息
     * @return 编号
     */
    Long createData(List<FactoryDataDO> factoryDataDOS);

    /**
     * 更新工厂出库入库数据
     *
     * @param updateReqVO 更新信息
     */
    void updateData(@Valid FactoryDataSizeCreateReqVO updateReqVO);

    /**
     * 删除工厂出库入库数据
     *
     * @param factoryId 工厂编号
     * @param date 日期
     */
    void deleteData(Long factoryId, String date);

    /**
     * 获得工厂出库入库数据
     *
     * @param factoryId 工厂编号
     * @param date 日期
     * @return 工厂出库入库数据
     */
    FactoryDataSizeDetailRespVO getData(Long factoryId, String date);

    /**
     * 获得工厂出库入库数据分页
     *
     * @param pageReqVO 分页查询
     * @return 工厂出库入库数据分页
     */
    PageResult<FactoryDataDetailRespVO> getDataPage(FactoryDataPageReqVO pageReqVO);

    /**
     * 获得工厂总数 总览
     * @return
     */
    FactoryDataTotalVO getDataTotal(Long factoryId, LocalDate[] date);

    /**
     * 获取当天的所有数据
     *
     * @return
     */
    List<FactoryDataDO> getTodayInboundAndOutboundOverview(Date time);

    /**
     * 根据日期和工厂过滤
     *
     * @param factoryId
     * @param beginDay
     * @param endDay
     * @return
     */
    List<FactoryDataDO> getByTimeAndFactoryId(Long factoryId, Date beginDay, Date endDay);

    /**
     * 根据日期和工厂过滤
     *
     * @param factoryId 工厂编号
     * @param date 日期
     * @param type 出入库类型
     * @return 出入库数据
     */
    FactoryDataDO getListByType(Long factoryId, LocalDate date, Integer type);

    /**
     * 导入出入库数据
     * @param list 数据
     * @return 结果
     */
    FactoryImportRespVO importUserList(List<FactoryDataImportVO> list, Integer type);
}