package cn.iocoder.yudao.module.smartfactory.service.factorydata;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.*;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeCreateReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeDetailRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataCreateReqVO;
import cn.iocoder.yudao.module.smartfactory.convert.packageData.PackageDataConvert;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorydata.FactoryDataDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorydata.FactoryDataSizeDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factoryinfo.FactoryInfoDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.packagedata.PackageDataDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.factorydata.FactoryDataMapper;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.factorydata.FactoryDataSizeMapper;
import cn.iocoder.yudao.module.smartfactory.service.factoryinfo.FactoryInfoService;
import cn.iocoder.yudao.module.smartfactory.service.packageInfo.PackageService;
import cn.iocoder.yudao.module.smartfactory.service.packagedata.PackageDataService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertMap;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.*;


/**
 * 工厂出库入库数据 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class FactoryDataServiceImpl implements FactoryDataService {

    @Resource
    private FactoryDataMapper dataMapper;

    @Resource
    @Lazy // 循环依赖，避免报错
    private FactoryDataSizeService dataSizeService;

    @Resource
    private FactoryInfoService factoryInfoService;

    @Resource
    private PackageDataService packageDataService;

    @Resource
    private FactoryDataSizeMapper factoryDataSizeMapper;

    @Resource
    private PackageService packageService;

    @Override
    public Long createData(List<FactoryDataDO> factoryDataDOS) {

        dataMapper.insertBatch(factoryDataDOS);

        // 返回
        return (long) factoryDataDOS.size();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateData(FactoryDataSizeCreateReqVO updateReqVO) {

        // 修改 规格明细数据
        dataSizeService.updateData(updateReqVO);

        //校验 出入库统计数据是否存在
        List<FactoryDataDO> factoryDataDOS = dataMapper.selectListByFactoryAndDate(updateReqVO.getFactoryId(), updateReqVO.getDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        if (factoryDataDOS != null && !factoryDataDOS.isEmpty()) {

            PackageDataCreateReqVO createReqVO = new PackageDataCreateReqVO();
            createReqVO.setPackageData(updateReqVO.getPackageData());
            createReqVO.setFactoryId(updateReqVO.getFactoryId());
            createReqVO.setPackageDate(updateReqVO.getDate());

            //获得 打包线数据对象
            List<PackageDataDO> packageDataDOS = PackageDataConvert.INSTANCE.convertList(createReqVO);
            // 更新
            factoryDataDOS.forEach(data -> {

                switch (data.getDataType()) {

                    //出库数据
                    case 1:
                        data.setTotalNum(updateReqVO.getItems().stream().mapToInt(FactoryDataSizeSaveReqVO::getOutNum).sum());
                        data.setDamageNum(updateReqVO.getItems().stream().mapToInt(FactoryDataSizeSaveReqVO::getOutDamageNum).sum());
                        break;
                    //入库数据
                    case 2:
                        Integer damageNum = packageDataDOS.stream().mapToInt(PackageDataDO::getDamageNum).sum()
                                + updateReqVO.getItems().stream().mapToInt(FactoryDataSizeSaveReqVO::getInDamageNum).sum();

                        data.setTotalNum(packageDataDOS.stream().mapToInt(info -> info.getNum() + info.getAutoNum()).sum());
                        data.setDamageNum(damageNum);
                        data.setSquareNum(packageDataDOS.stream().map(PackageDataDO::getSquareNum).reduce(BigDecimal.ZERO, BigDecimal::add));
                }
            });

            //更新 出入库统计数据
            dataMapper.updateBatch(factoryDataDOS);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteData(Long factoryId, String date) {

        //删除出入库 规格数据
        dataSizeService.deleteData(factoryId, date);

        //删除打包线数据
        packageDataService.deletePackageData(factoryId, date);
    }

    @Override
    public FactoryDataSizeDetailRespVO getData(Long factoryId, String date) {

        FactoryDataSizeDetailRespVO respVO = new FactoryDataSizeDetailRespVO();
        respVO.setFactoryId(factoryId);
        respVO.setDate(LocalDate.parse(date));

        //获得 规则明细数据
        List<FactoryDataInfoRespVO> infoRespVOS = dataSizeService.getDetailList(factoryId, new LocalDate[]{LocalDate.parse(date), LocalDate.parse(date)}, null);

        List<FactoryDataSizeDetailRespVO.InItem> packageInfo = infoRespVOS.stream()
                .filter(info -> (info.getInDamageNum() != null && info.getInNum() == null) || info.getNum() != null || info.getAutoNum() != null)
                .map(info -> BeanUtil.toBean(info, FactoryDataSizeDetailRespVO.InItem.class))
                .collect(Collectors.toList());

        List<FactoryDataSizeDetailRespVO.InItem> inItems = infoRespVOS.stream()
                .filter(info -> info.getInNum() != null)
                .map(info ->  {
                    FactoryDataSizeDetailRespVO.InItem inItem = BeanUtil.toBean(info, FactoryDataSizeDetailRespVO.InItem.class);
                    inItem.setIsEnable(1);
                    return inItem;
                })
                .collect(Collectors.toList());
        inItems.addAll(packageInfo);
        respVO.setInItems(inItems);

        respVO.setOutItems(
                infoRespVOS.stream()
                        .filter(info -> info.getOutNum() != null || info.getOutDamageNum() != null)
                        .map(info -> BeanUtil.toBean(info, FactoryDataSizeDetailRespVO.OutItem.class))
                        .collect(Collectors.toList()));

        return respVO;
    }

    @Override
    public PageResult<FactoryDataDetailRespVO> getDataPage(FactoryDataPageReqVO pageReqVO) {

        //获得 出入库数据分页列表
        IPage<FactoryDataInfoRespVO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        IPage<FactoryDataInfoRespVO> page = dataMapper.selectDetailPage(mpPage, pageReqVO);

        List<FactoryDataInfoRespVO> respVOS = page.getRecords();
        List<FactoryDataDetailRespVO> toDayData = new ArrayList<>();
        if (!respVOS.isEmpty()) {

            // 按 工厂id和日期 进行分组
            Map<Long, Map<LocalDate, List<FactoryDataInfoRespVO>>> groupedByFactoryIdAndDate = respVOS.stream()
                    .collect(Collectors.groupingBy(FactoryDataInfoRespVO::getFactoryId,
                            Collectors.groupingBy(FactoryDataInfoRespVO::getDate)));

            //获得工厂信息列表
            Map<Long, FactoryInfoDO> factoryMaps = convertMap(factoryInfoService.getFactoryList(groupedByFactoryIdAndDate.keySet())
                    , FactoryInfoDO::getId);
            groupedByFactoryIdAndDate.forEach((factoryId, dateMap) -> {

                //按日期倒叙 并遍历
                dateMap.entrySet().stream().sorted(Map.Entry.<LocalDate, List<FactoryDataInfoRespVO>>comparingByKey().reversed()).forEach((date) -> {

                    FactoryDataDetailRespVO dataDetailRespVO = new FactoryDataDetailRespVO();

                    dataDetailRespVO.setFactoryId(factoryId);
                    dataDetailRespVO.setFactoryName(factoryMaps.get(factoryId).getName());
                    dataDetailRespVO.setDate(date.getKey());
                    //设置入库总数
                    int inSum = date.getValue().stream()
                            .map(info -> {
                                Integer num = Optional.ofNullable(info.getNum()).orElse(0);
                                Integer autoNum = Optional.ofNullable(info.getAutoNum()).orElse(0);
                                Integer inNUm = Optional.ofNullable(info.getInNum()).orElse(0);
                                return num + autoNum + inNUm;
                            })
                            .mapToInt(Integer::intValue).sum();
                    dataDetailRespVO.setInTotalNum(inSum == 0 ? null : inSum);

//                    //设置入库平方数
//                    dataDetailRespVO.setSquareNum(date.getValue().stream().map(FactoryDataInfoRespVO::getSquareNum)
//                            .filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add));

                    //设置出库总数
                    int outTotalNum = date.getValue().stream().filter(info -> info.getOutNum() != null)
                            .mapToInt(FactoryDataInfoRespVO::getOutNum).sum();
                    dataDetailRespVO.setOutTotalNum(outTotalNum == 0 ? null : outTotalNum);

                    //设置 出入库破损总数
                    int sum = date.getValue().stream().map(info -> {
                        FactoryDataInfoRespVO respVO = BeanUtils.toBean(info, FactoryDataInfoRespVO.class);
                        respVO.setInDamageNum(Optional.ofNullable(info.getInDamageNum()).orElse(0));
                        respVO.setOutDamageNum(Optional.ofNullable(info.getOutDamageNum()).orElse(0));
                        return respVO;
                    }).mapToInt(info -> info.getInDamageNum() + info.getOutDamageNum()).sum();

                    dataDetailRespVO.setDamageNum(sum == 0 ? null : sum);

                    date.getValue().forEach(info -> {
                        int num = Optional.ofNullable(info.getNum()).orElse(0);
                        int autoNum = Optional.ofNullable(info.getAutoNum()).orElse(0);
                        int inNum = Optional.ofNullable(info.getInNum()).orElse(0);
                        info.setInNum((num + autoNum + inNum) == 0 ? null : (num + autoNum + inNum));
                    });
                    dataDetailRespVO.setItems(date.getValue());

                    if (date.getValue().get(0).getCreateTime().toLocalDate().equals(LocalDate.now())) {

                        //设置 可编辑状态
                        dataDetailRespVO.setIsEdit(1);
                    }

                    toDayData.add(dataDetailRespVO);
                });
            });
        }

        return new PageResult<>(toDayData, page.getTotal());
    }

    @Override
    public FactoryDataTotalVO getDataTotal(Long factoryId, LocalDate[] date) {

        return dataMapper.selectDataSum(factoryId, date) == null ? new FactoryDataTotalVO() : dataMapper.selectDataSum(factoryId, date);
    }

    @Override
    public List<FactoryDataDO> getTodayInboundAndOutboundOverview(Date time) {
        String timeStr = null;
        if (time != null) {
            timeStr = DateUtils.dateFormat(time, "yyyy-MM-dd");
        }
        return dataMapper.selectList(new LambdaQueryWrapperX<FactoryDataDO>()
                .eq(timeStr != null, FactoryDataDO::getDate, timeStr));
    }

    @Override
    public List<FactoryDataDO> getByTimeAndFactoryId(Long factoryId, Date beginDay, Date endDay) {
        return dataMapper.selectList(new LambdaQueryWrapperX<FactoryDataDO>()
                .ge(FactoryDataDO::getDate, beginDay)
                .le(FactoryDataDO::getDate, endDay)
                .eq(FactoryDataDO::getFactoryId, factoryId));
    }

    @Override
    public FactoryDataDO getListByType(Long factoryId, LocalDate date, Integer type) {

        return dataMapper.selectListByType(factoryId, date, type);
    }

    @Override
    @Transactional(rollbackFor = Exception.class) // 添加事务，异常则回滚所有导入
    public FactoryImportRespVO importUserList(List<FactoryDataImportVO> list, Integer type) {

        if (CollUtil.isEmpty(list)) {
            throw exception(FACTORY_IMPORT_LIST_IS_EMPTY);
        }

        FactoryImportRespVO respVO = FactoryImportRespVO.builder().createUsernames(new ArrayList<>()).updateUsernames(new ArrayList<>()).failureUsernames(new LinkedHashMap<>()).build();

        list.forEach(data -> {

            //校验，判断是否有不符合的原因
            Long factoryId = null;
            try {
                factoryId = validateFactoryForCreate(data, type);
            } catch (Exception ex) {

                respVO.getFailureUsernames().put(data.getFactoryName() + "-" + data.getDate(), ex.getMessage());
                return;
            }

            // 获取 规格ID
            Long sizeId = Long.valueOf(data.getSizeName().split(":")[0]);
            // 获取 规格名称
            String sizeName = data.getSizeName().split(":")[1];

//            PackageDO packageDO = packageService.getPackageByFactoryIdAndSize(factoryId, sizeId);
//            if (packageDO != null && (data.getNum() != null || data.getAutoNum() != null)) {
//
//                PackageDataDO packageDataDO = BeanUtils.toBean(packageDO, PackageDataDO.class);
//                packageDataDO.setId(null);
//                packageDataDO.setPackageDate(data.getDate());
//                packageDataDO.setPackageId(packageDO.getId());
//                packageDataDO.setPackageName(packageDO.getName());
//                packageDataDO.setSizeName(sizeName);
//                packageDataDO.setNum(data.getNum());
//                packageDataDO.setAutoNum(data.getAutoNum());
//                packageDataDO.setDamageNum(data.getDamageNum());
//                packageDataDO.setCreator(getLoginUserId().toString());
//                packageDataDO.setUpdater(null);
//                packageDataDO.setCreateTime(LocalDateTime.now());
//
//                //计算平方数
//                String[] size = sizeName.split("\\*");
//                BigDecimal squareNum = BigDecimal.valueOf((double) Integer.parseInt(size[0]) / 1000)
//                        .multiply(BigDecimal.valueOf((double) Integer.parseInt(size[1]) / 1000))
//                        .multiply(BigDecimal.valueOf(packageDataDO.getSizePieces()))
//                        .multiply(BigDecimal.valueOf(Integer.sum(packageDataDO.getNum(), packageDataDO.getAutoNum())));
//
//                //设置平方数
//                packageDataDO.setSquareNum(squareNum);
//
//                // 插入打包线数据
//                packageDataService.create(packageDataDO);
//            }

            List<FactoryDataSizeDO> factoryDataSizeDOs = new ArrayList<>();

            if (data.getNum() != null) {

                FactoryDataSizeDO dataSizeDO = new FactoryDataSizeDO();
                //设置规格信息
                dataSizeDO.setSizeId(sizeId);
                dataSizeDO.setSizeName(sizeName);
                //设置工厂和日期
                dataSizeDO.setFactoryId(factoryId);
                dataSizeDO.setDate(data.getDate());
                // 根据type 设置入库数还是出库数
                dataSizeDO.setType(type == 1 ? 4 : 2);
                dataSizeDO.setNum(data.getNum());

                factoryDataSizeDOs.add(dataSizeDO);
            }

            if (data.getDamageNum() != null) {

                FactoryDataSizeDO dataSizeDamageDO = new FactoryDataSizeDO();
                //设置规格信息
                dataSizeDamageDO.setSizeId(sizeId);
                dataSizeDamageDO.setSizeName(sizeName);
                //设置工厂和日期
                dataSizeDamageDO.setFactoryId(factoryId);
                dataSizeDamageDO.setDate(data.getDate());
                // 根据type 设置入库破损还是出库破损
                dataSizeDamageDO.setType(type == 1 ? 1 : 3);
                dataSizeDamageDO.setNum(data.getDamageNum());

                factoryDataSizeDOs.add(dataSizeDamageDO);
            }

            factoryDataSizeMapper.insertBatch(factoryDataSizeDOs);

            respVO.getCreateUsernames().add(data.getFactoryName() + "-" + data.getDate());
        });

        return respVO;
    }

    private Long validateFactoryForCreate(FactoryDataImportVO data, Integer type) {

        String factoryName = data.getFactoryName();
        String sizeName = data.getSizeName();
        LocalDate date = data.getDate();

        if (factoryName == null) {
            throw exception(FACTORY_INFO_NOT_EXISTS);
        }

        if (sizeName == null) {
            throw exception(SIZE_NOT_EXISTS);
        }

        FactoryInfoDO factoryInfoDO = factoryInfoService.getFactoryByName(factoryName);

        List<Integer> dataType = new ArrayList<>();
        if (data.getNum() != null && data.getDamageNum() != null) {
            dataType = type == 1 ? Arrays.asList(1,4) : Arrays.asList(2,3);
        }

        if (data.getNum() != null && data.getDamageNum() == null) {
            dataType = type == 1 ? Collections.singletonList(4) : Collections.singletonList(2);
        }

        if (data.getNum() == null && data.getDamageNum() != null) {
            dataType = type == 1 ? Collections.singletonList(1) : Collections.singletonList(3);
        }

        // 获取 规格ID
        Long sizeId = Long.valueOf(sizeName.split(":")[0]);
        Long count = factoryDataSizeMapper.selectCountByType(factoryInfoDO.getId(), sizeId, date, dataType);
//        List<PackageDataDO> packageDOs = packageDataService.getListByFactoryIdAndDate(factoryInfoDO.getId(), date.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));

        if (count > 0L) {
            throw exception(PACKAGE_DATA_EXISTS);
        }

        return factoryInfoDO.getId();
    }
}