package cn.iocoder.yudao.module.smartfactory.service.factorydata;

import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.FactoryDataInfoRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.sizeData.FactoryDataSizeCreateReqVO;

import javax.validation.Valid;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 工厂出库入库规格数据 Service 接口
 *
 * @author 符溶馨
 */
public interface FactoryDataSizeService {

    /**
     * 创建工厂规格数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createData(@Valid FactoryDataSizeCreateReqVO createReqVO);

    /**
     * 更新工厂规格数据
     *
     * @param updateReqVO 更新信息
     */
    void updateData(@Valid FactoryDataSizeCreateReqVO updateReqVO);

    /**
     * 删除工厂规格数据
     *
     * @param factoryId 工厂编号
     * @param date 日期
     */
    void deleteData(Long factoryId, String date);

    /**
     * 获得出入库规格数据 列表
     * @return 出入库规格数据
     */
    List<FactoryDataInfoRespVO> getDetailList(Long factoryId, LocalDate[] date, LocalDateTime[] createTime);

    /**
     * 获得指定时间段内 出入库规格数据之和
     * @param dateTimes 时间区间
     * @return 出入库规格数据之和
     */
    List<FactoryDataInfoRespVO> getSUMByDateTimes(LocalDateTime[] dateTimes);
}
