package cn.iocoder.yudao.module.smartfactory.service.factoryinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factoryinfo.vo.FactoryInfoPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factoryinfo.vo.FactoryInfoSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factoryinfo.vo.FactoryUpdateStatusReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.screendata.factory.vo.FactoryRollDataRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.screendata.factory.vo.ProvincesDataRespVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factoryinfo.FactoryInfoDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 工厂信息 Service 接口
 *
 * @author 姚君
 */
public interface FactoryInfoService {

    /**
     * 创建工厂信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createFactoryInfo(@Valid FactoryInfoSaveReqVO createReqVO);

    /**
     * 更新工厂信息
     *
     * @param updateReqVO 更新信息
     */
    void updateFactoryInfo(@Valid FactoryInfoSaveReqVO updateReqVO);

    /**
     * 删除工厂信息
     *
     * @param id 编号
     */
    void deleteFactoryInfo(Long id);

    /**
     * 获得工厂信息
     *
     * @param id 编号
     * @return 工厂信息
     */
    FactoryInfoDO getFactoryInfo(Long id);

    /**
     * 获得工厂信息列表
     *
     * @return 工厂信息
     */
    List<FactoryInfoDO> getFactoryList();

    /**
     * 获得工厂信息列表
     *
     * @param ids 编号
     * @return 工厂信息
     */
    List<FactoryInfoDO> getFactoryList(Collection<Long> ids);

    /**
     * 获得工厂信息列表
     *
     * @param factoryId 编号
     * @return 工厂信息
     */
    List<FactoryInfoDO> getFactoryList(Long factoryId, String name);

    /**
     * 获得类型为工厂的工厂信息列表
     * @return 工厂列表
     */
    List<FactoryInfoDO> getFactoryListByType();

    /**
     * 获得工厂信息分页
     *
     * @param pageReqVO 分页查询
     * @return 工厂信息分页
     */
    PageResult<FactoryInfoDO> getFactoryInfoPage(FactoryInfoPageReqVO pageReqVO);

    /**
     * 省市区类型 + code 搜索工厂
     *
     * @param code
     * @return
     */
    List<ProvincesDataRespVO> getProvincesData(String code);

    /**
     * 大屏数据 - 工厂基础滚动数据
     *
     * @return
     */
    List<FactoryRollDataRespVO> getFactoryRollData();

    /**
     * 工厂列表
     *
     * @return
     */
    List<FactoryInfoDO> list();

    /**
     * 根据工厂ID获取天气代码。
     *
     * @param factoryId 工厂的唯一标识符，类型为Long。
     * @return 返回与给定工厂ID相关联的天气代码，类型为String。
     */
    String getWeatherCodeByFactoryId(Long factoryId, String areaCode);

    /**
     * 获得类型为工厂的工厂信息列表
     * @return 工厂列表
     */
    List<FactoryInfoDO> getFactoryListByAuthority();

    /**
     * 获得指定工厂信息
     * @param factoryName 工厂简称
     * @return 工厂信息
     */
    FactoryInfoDO getFactoryByName(String factoryName);

    /**
     * 修改 工厂状态
     * @param updateReqVO 更新信息
     */
    void updateFactoryStatus(FactoryUpdateStatusReqVO updateReqVO);
}
