package cn.iocoder.yudao.module.smartfactory.service.packageInfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packageInfo.vo.PackagePageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packageInfo.vo.PackageRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packageInfo.vo.PackageSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.packageInfo.PackageDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 打包线 Service 接口
 *
 * @author 符溶馨
 */
public interface PackageService {

    /**
     * 创建打包线
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPackage(@Valid PackageSaveReqVO createReqVO);

    /**
     * 更新打包线
     *
     * @param updateReqVO 更新信息
     */
    void updatePackage(@Valid PackageSaveReqVO updateReqVO);

    /**
     * 删除打包线
     *
     * @param id 编号
     */
    void deletePackage(Long id);

    /**
     * 获得打包线
     *
     * @param id 编号
     * @return 打包线
     */
    PackageDO getPackage(Long id);

    /**
     * 获得指定工厂的打包线列表
     * @param factoryId 工厂编号
     * @return 打包线信息列表
     */
    List<PackageRespVO> getPackageByFactoryId(Long factoryId);

    /**
     * 获得指定工厂的打包线
     * @param factoryId 工厂编号
     * @param sizeId 规格编号
     * @return 打包线信息
     */
    PackageDO getPackageByFactoryIdAndSize(Long factoryId, Long sizeId);

    /**
     * 获得打包线分页
     *
     * @param pageReqVO 分页查询
     * @return 打包线分页
     */
    PageResult<PackageRespVO> getPackagePage(PackagePageReqVO pageReqVO);
}