package cn.iocoder.yudao.module.smartfactory.service.packagedata;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataCreateReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataRespVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorydata.FactoryDataDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.packagedata.PackageDataDO;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * 打包线数据 Service 接口
 *
 * @author 艾楷
 */
public interface PackageDataService {

    /**
     * 创建打包线数据
     *
     * @param createReqVO 创建信息
     */
    void create(@Valid PackageDataDO createReqVO);

    /**
     * 创建打包线数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPackageData(@Valid PackageDataCreateReqVO createReqVO);

    /**
     * 更新打包线数据
     *
     * @param updateReqVO 更新信息
     */
    void updatePackageData(@Valid PackageDataCreateReqVO updateReqVO);

    /**
     * 删除打包线数据
     *
     * @param factoryId 工厂编号
     * @param date 打包日期
     */
    void deletePackageData(Long factoryId, String date);

    /**
     * 获得打包线数据分页
     *
     * @param pageReqVO 分页查询
     * @return 打包线数据分页
     */
    PageResult<PackageDataRespVO> getPackageDataPage(PackageDataPageReqVO pageReqVO);

    /**
     * 列表
     *
     * @param factoryId
     * @return
     */
    List<PackageDataDO> getFactoryPackLineInfo(Long factoryId);

    /**
     * 列表 - 时间参数
     *
     * @param factoryId
     * @param beginTime
     * @param endTime
     * @return
     */
    List<PackageDataDO> getFactoryPackLineInfoByTime(Long factoryId, Date beginTime, Date endTime);

    /**
     * 获得指定时间段的打包线数据列表
     * @param dateTimes 时间区间
     * @return 打包线数据列表
     */
    List<FactoryDataDO> getDataListByDateTime(LocalDateTime[] dateTimes);

    /**
     * 获得指定 工厂和打包日期的 打包线数据列表
     * @param factoryId 工厂编号
     * @param date 打包线日期
     * @return 打包线数据列表
     */
    List<PackageDataDO> getListByFactoryIdAndDate(Long factoryId, String date);
}